import os, sys
import numpy as np
import cv2
import time
import pytesseract
from textblob import TextBlob
import pyttsx3
from imutils.object_detection import non_max_suppression
from skew import deskew
def speak(st):
    engine = pyttsx3.init()
    voices = engine.getProperty('voices')
    engine.setProperty('rate', 90) 
    engine.setProperty('voice', voices[11].id)
    engine.say(st)
    engine.runAndWait()
    engine.stop()



def processimg(img):
#    print(img)
    gray = cv2.cvtColor(img, cv2.COLOR_RGB2GRAY)
    img_bin = cv2.adaptiveThreshold(gray, 255,cv2.ADAPTIVE_THRESH_GAUSSIAN_C, cv2.THRESH_BINARY,11,2)
    #gray = cv2.bitwise_not(img_bin)
    kernel = np.ones((2, 1), np.uint8)
    img = cv2.erode(gray, kernel, iterations=1)
    img = cv2.dilate(img, kernel, iterations=1)
    return img

def east_detect(image):
    outTex = ""
    image = deskew(image)
    cv2.imwrite("deskew.jpg",image)
    image = processimg(image)
    
    layerNames = [
    	"feature_fusion/Conv_7/Sigmoid",
    	"feature_fusion/concat_3"]
    
    orig = image.copy()
    
    if len(image.shape) == 2:
        image = cv2.cvtColor(image, cv2.COLOR_GRAY2RGB)
    
    (H, W) = image.shape[:2]
    
    # set the new width and height and then determine the ratio in change
    # for both the width and height: Should be multiple of 32
    (newW, newH) = (320, 320)
    
    rW = W / float(newW)
    rH = H / float(newH)
    
    # resize the image and grab the new image dimensions
    image = cv2.resize(image, (newW, newH))
    
    (H, W) = image.shape[:2]
    
    net = cv2.dnn.readNet("model/frozen_east_text_detection.pb")
    
    blob = cv2.dnn.blobFromImage(image, 1.0, (W, H),
    	(123.68, 116.78, 103.94), swapRB=True, crop=False)
    
    start = time.time()
    
    net.setInput(blob)
    
    (scores, geometry) = net.forward(layerNames)
    
    (numRows, numCols) = scores.shape[2:4]
    rects = []
    confidences = []
    # loop over the number of rows
    for y in range(0, numRows):
        # extract the scores (probabilities), followed by the geometrical
        # data used to derive potential bounding box coordinates that
        # surround text
        scoresData = scores[0, 0, y]
        xData0 = geometry[0, 0, y]
        xData1 = geometry[0, 1, y]
        xData2 = geometry[0, 2, y]
        xData3 = geometry[0, 3, y]
        anglesData = geometry[0, 4, y]
    
        for x in range(0, numCols):
    		# if our score does not have sufficient probability, ignore it
            # Set minimum confidence as required
            if scoresData[x] < 0.3:
                continue
    		# compute the offset factor as our resulting feature maps will
            #  x smaller than the input image
            (offsetX, offsetY) = (x * 4.0, y * 4.0)
            # extract the rotation angle for the prediction and then
            # compute the sin and cosine
            angle = anglesData[x]
            cos = np.cos(angle)
            sin = np.sin(angle)
            # use the geometry volume to derive the width and height of
            # the bounding box
            h = xData0[x] + xData2[x]
            w = xData1[x] + xData3[x]
            h= h*1.2
            w = w*1.2
      	    # compute both the starting and ending (x, y)-coordinates for
            # the text prediction bounding box
            endX = int(offsetX + (cos * xData1[x]) + (sin * xData2[x]))
            endY = int(offsetY - (sin * xData1[x]) + (cos * xData2[x]))
            startX = int(endX - w)
            startY = int(endY - h)
            # add the bounding box coordinates and probability score to
            # our respective lists
            rects.append((startX, startY, endX, endY))
            confidences.append(scoresData[x])
                        
    boxes = non_max_suppression(np.array(rects), probs=confidences)
    # loop over the bounding boxes	  
    flaff = 0
    for (startX, startY, endX, endY) in boxes:
    	# scale the bounding box coordinates based on the respective
    	# ratios
    	startX = int(startX * rW)
    	startY = int(startY * rH)
    	endX = int(endX * rW)
    	endY = int(endY * rH)
    	hei = orig.shape[0]
    	wid = orig.shape[1]

        
    	# draw the bounding box on the image
    	sY = max(1,startY-2)
    	eY = min(hei-2,endY+2)

    	sX = max(1,startX-20)
    	eX = min(wid-2,endX+20)
    #	print (str(eY) + " " + str(sY) + " " +str(eX) + " " + str(sX))
    	c = orig[sY:eY,sX:eX]
    	cv2.imwrite("im" + str(flaff) + ".jpg",c)
    	outTex = outTex + " " +  pytesseract.image_to_string(c, config='--psm 6')
#    	print(text)
    	flaff = flaff+1
#    	cv2.rectangle(orig, (startX, startY), (endX, endY), (0, 255, 0), 2)
	

    textBlb = TextBlob(outTex)            # Making our first textblob
    textCorrected = textBlb.correct()   # Correcting the text
    noun = [ word for (word,tag) in  textCorrected.tags if tag == "NNP"]
    return noun
   # for i in noun:
    #    speak(noun)
    #print(textCorrected.tags)
   # print(time.time() - start)
   # return orig

#image = cv2.imread("test.jpg")

#out_image = east_detect(image)

#cv2.imwrite("sample_output.jpg", out_image)
